import sys
import pandas as pd


def build_samples(data_file):
    data = pd.read_csv(data_file, skipinitialspace=True)
    headers = list(data)
    samples = []

    sample = pd.DataFrame()
    current = 0
    next = 1
    max = len(data.columns)
    while next <= max:
        # removing NaN rows at the very end of the file
        col = data.iloc[:, current].dropna()
        sample[headers[current]] = col
        if (next == max) or ('gene_symbol' in headers[next]):
            samples.append(sample)
            sample = pd.DataFrame()
        current += 1
        next += 1
    return samples


def capitalise(sample):
    samp_series = sample.iloc[:, 0]
    samp_series = samp_series.str.upper()
    return pd.concat([samp_series, sample.iloc[:, 1:len(sample)]], axis=1)


def get_mean_of_dups(sample):
    sample = sample.groupby('gene_symbol').mean().reset_index()
    return sample

# master function
def input(inputfile, output, nonan, sort, round):
    print('Building database structure...')
    samples = build_samples(inputfile)
    if  not sort:
        print('Capitalising genes...')
        for i in range(len(samples)):
            samples[i] = capitalise(samples[i])
        print('Sorting...')
        for i in range(len(samples)):
            samples[i] = samples[i].sort_values([list(samples[i])[0]])
    all = samples[0].copy(deep=True)
    all = get_mean_of_dups(all)
    for i in range(1, len(samples)):
        s = samples[i]
        if 'gene_symbol' in s.columns[0]:
            cols = s.columns.values
            cols[0] = 'gene_symbol'
            s.columns = cols
        s = get_mean_of_dups(s)
        all = all.merge(s, how='outer', on='gene_symbol', copy=False)
    all = all.sort_values([list(all)[0]])
    if nonan:
        print('Excluding rows with missing entries...')
        all = all.dropna()
    if round:
        print('Rounding...')
        all = all.round(round)
    all.to_csv((output+'.csv'), index=False)
    print('Done.')



import sys
import numpy as np
import pandas as pd
# import plotly.plotly as py
import plotly.graph_objs as go
from plotly.offline import plot


def verify_inputs(inpt_arr, headers):
    cols = []
    for inpt in inpt_arr:
        if not inpt.isdigit():
            for j in range(len(headers)):
                valid = False
                if inpt == headers[j]:
                    cols.append(j)
                    valid = True
                    break
            if not valid:
                print('your input \'%s\' was invalid! E0' % (inpt))
                sys.exit()
        elif (int(inpt) > 1) & (int(inpt) <= len(headers)):
                cols.append(inpt)
        else:
            print('your input \'%s\' was invalid! E1' % (inpt))
            sys.exit()
    for i in range(len(cols)):
        cols[i] = int(cols[i]) - 1
    return cols


def gen_graph_array(data, index_arr):
    output = []  # array of DataFrames
    for i in range(len(index_arr)):
        sample_frame = pd.DataFrame()
        for index in index_arr[i]:
            sample_frame[str(list(data)[index])] = data.iloc[:, index].values
        name = sample_frame.columns[0]
        for col in range(1, len(sample_frame.columns)):
            name = name + ', ' + sample_frame.columns[col]
        if len(sample_frame.columns) > 1:
            sample_frame = sample_frame.mean(axis=1)
        sample_frame = sample_frame.squeeze()
        sample_frame = sample_frame.rename(name)
        output.append(sample_frame.squeeze())  # Append only series
    return output


def remove_na_rows(series_arr, gene_names, colour, label):
    output = []
    for i in range(1, len(series_arr)):
        na_frame = pd.DataFrame()
        na_frame['names'] = gene_names
        na_frame[series_arr[0].name] = series_arr[0]  # append control
        na_frame[series_arr[i].name] = pd.Series(series_arr[i])  # append sample
        if colour is not None:
            na_frame[colour.name] = colour
        if label is not None:
            na_frame['label'] = label
        na_frame = na_frame.dropna(axis=0, how='any')
        output.append(na_frame)
    return(output)


def gen_graphs(samples, gene_names, alpha, colour, label, nolegend):    
    if len(samples) < 2:
        print('Something went wrong! E3')
        sys.exit()
    graph_sets = remove_na_rows(samples, gene_names, colour, label)
    count = 1
    for gs in graph_sets:
        print('Sample ' + str(count) + ':')
        count = count + 1
        graphs = []
        print('Computing main graph...')
        graph = go.Scattergl(
            x=gs.iloc[:, 1].values,
            y=gs.iloc[:, 2].values,
            mode='markers',
            text=gs.iloc[:, 0].values)
        graphs.append(graph)

        if label is not None:
            print('Computing 10 largest and 10 smallest values of given label input...')
            sorted = gs.sort_values(['label'])
            largest = sorted.iloc[0:10,]
            smallest = sorted.iloc[-10:,]
            graph = go.Scatter(
                x=largest.iloc[:, 1].values,
                y=largest.iloc[:, 2].values,
                mode = 'markers+text',
                name = 'largest',
                marker = dict(color = 'rgba(0,0,152,1)'),
                text=largest.iloc[:, 0].values,
                textposition='bottom',
                textfont=dict(
                    family='sans serif',
                    size=18,
                    color='#cf3523'
            ))
            graphs.append(graph)
            graph = go.Scatter(
                x=smallest.iloc[:, 1].values,
                y=smallest.iloc[:, 2].values,
                mode = 'markers+text',
                name = 'smallest',
                marker = dict(color = 'rgba(0,0,152,1)'),
                text=smallest.iloc[:, 0].values,
                textposition='bottom',
                textfont=dict(
                    family='sans serif',
                    size=18,
                    color='#1f77b4'
            ))
            graphs.append(graph)

        if colour is not None:
            graphs.pop(0)  # Removing main graph, since it will be represented
            # by the pvalue comparisons anyway.
            print('Computing pvalues relative to alpha...')
            more = gs[gs.iloc[:,3] > alpha]
            less = gs[gs.iloc[:,3] <= alpha]
            graph = go.Scatter(
                x=more.iloc[:, 1].values,
                y=more.iloc[:, 2].values,
                mode = 'markers',
                name = 'Above ' + str(alpha),
                marker = dict(color = 'rgba(0,0,0,1)'),
                text=more.iloc[:, 0].values)
            graphs.append(graph)
            graph = go.Scatter(
                x=less.iloc[:, 1].values,
                y=less.iloc[:, 2].values,
                mode = 'markers',
                name = 'Below ' + str(alpha),
                marker = dict(color = 'rgba(255,0,0,1)'),
                text=less.iloc[:, 0].values)
            graphs.append(graph)

        
        layout = go.Layout(
            title=('Sample ' + gs.columns.values[2]),
            xaxis=dict(title='Control: ' + gs.columns.values[1]),
            yaxis=dict(title=gs.columns.values[2]),
            showlegend=nolegend
        )

        fig = go.Figure(data=graphs, layout=layout)
        plot(fig, filename='Sample_' + str(gs.columns.values[2]) + '.html', auto_open=False)


# master function
def input(inputfile, scatter, heat, alpha, colour, label, nolegend, control, samples):

    if scatter:
        if bool(alpha) ^ bool(colour):  # if alpha xor colour
            print('When using alpha or colour, you must specify *both* values!')
            system.exit()

        data = pd.read_csv(inputfile, skipinitialspace=True)

        index_arr = []
        index_arr.append(verify_inputs(control.split(','), list(data)))
        for sample in samples:
            index_arr.append(verify_inputs(sample.split(','), list(data)))

        graph_arr = gen_graph_array(data, index_arr)
        gene_symbols = np.array(data.iloc[:, 0].values).tolist()

        # Extra stuff
        if colour:
            col_arr = verify_inputs(colour.split(','), list(data))
            colour = gen_graph_array(data, [col_arr])[0]
        if label:
            label_arr = verify_inputs(label.split(','), list(data))
            label = gen_graph_array(data, [label_arr])[0]


        gen_graphs(graph_arr, gene_symbols, alpha, colour, label, nolegend)
        
    if heat:
        print(' Heat not implemented yet.')
    print('Done.')




import argparse
import sys
from helixpc import group_genes, graph_genes, __version__
# import group_genes, graph_genes  # for development use.
# __version__ = 'CHANGETHIS'  # for development use.


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument('-v','--version', action='version', 
                        version='%(prog)s ' + __version__)
    subparsers = parser.add_subparsers(title='subcommands - Type \'helix.py' +
                                       ' [function] -h\' to find out more. It ' +
                                       ' is highly recommended that you read '+
                                       'the README.rst for guidance.',
                                       metavar='', dest='command')

    # group subcommand
    group_parser = subparsers.add_parser('group', help='Group a series of fold ' +
                                         'change samples into a sing,le file ' +
                                         'with entries for every gene.')
    group_parser.add_argument('input', help='input file name')
    group_parser.add_argument('output', nargs='?',
                              help='output file name (optional)',
                              default='output')
    group_parser.add_argument('-n', '--nonan', action='store_true',
                              help='specifies if rows including at least  ' +
                              'one missing entry should be omitted.')
    group_parser.add_argument('-y', '--yes', action='store_true',
                              help='specifies if the file is already ' +
                              'capitalised and sorted or not.')
    group_parser.add_argument('-r','--round', type=int,
                              help='specifies by how much the values ' +
                              'should be rounded by.')


    # graph subcommand
    graph_parser = subparsers.add_parser('graph', help='Graphs a database of ' +
                                         'genes according to given controls.' +
                                         ' Takes the output of the [group]' +
                                         ' function as input.')
    graph_parser.add_argument('input', help='input file name')
    graph_parser.add_argument('-s', '--scatter', action='store_true',
                              help='generates scatter graph(s)')
    graph_parser.add_argument('-he', '--heat', action='store_true',
                              help='generates heat graph(s)')
    graph_parser.add_argument('-a', '--alpha', type=float,
                              help='Determines the alpha value for scatter ' +
                              ' graphs. Everything underneath (including) this ' +
                              'value will be colored red.')
    graph_parser.add_argument('-p','--pvalue', help='Lists the column(s) to be ' +
                              'used for pvalues.')
    graph_parser.add_argument('-l', '--label', help='specifies which column ' +
                              'should be used for labelling the 10 with the ' +
                              'highest values and the 10 with the lowest values.')
    graph_parser.add_argument('-nl', '--no-legend', action='store_false', 
                              help='disables legend display', default=True)
    graph_parser.add_argument('control', help='specifies the control')
    graph_parser.add_argument('samples', nargs='+', help='specifies the ' +
                              'samples')



    args = parser.parse_args()

    # early error handling
    if not len(sys.argv) > 1:
        parser.print_help()
        sys.exit()
    if (args.input is not None and args.input[-4:] != '.csv'):
        print("Input file must be in the .csv format.")
        sys.exit()
    if args.command == 'graph':
        if args.heat is not True and args.scatter is not True:
            args.heat = True
            args.scatter = True
        graph_genes.input(args.input, args.scatter, args.heat, args.alpha, args.pvalue, 
                          args.label, args.no_legend, args.control, args.samples)
    else:
        group_genes.input(args.input, args.output, args.nonan, args.yes, args.round)


main()



from setuptools import setup
exec(open('helixpc/_version.py').read())

setup(
    name='helixpc',
    version=__version__,
    license='MIT',
    description='Automisation of graph generation for gene FC databases.',
    long_description=open('README.rst').read(),
    author='Anne-Laure Ehresmann',
    author_email='cathaspa@protonmail.com',
    url='https://github.com/Cathaspa/HelixPC',
    packages=['helixpc'],
    entry_points = {
        'console_scripts': ['helixpc=helixpc.cli:main']
        },
    install_requires=[
        'numpy',
        'pandas',
        'plotly',
    ]
)
